var __reflect = (this && this.__reflect) || function (p, c, t) {
    p.__class__ = c, t ? t.push(c) : t = [c], p.__types__ = p.__types__ ? t.concat(p.__types__) : t;
};
var SocketMgr = (function () {
    function SocketMgr() {
        this.debugUrl = '172.16.14.8';
        this.debugPort = 9001;
        this.releaseUrl = '172.16.14.8'; //
        this.releasePort = 9001; //
        this.heartBeatTimes = 0;
        this.connected = false;
        this.reconnecting = false;
        this.reconnectTimes = 0;
        this.connectedCBs = [];
        this.reconnectCBs = [];
        this.errorCBs = [];
        this.needReconnect = false;
        this.hideLoading = true;
    }
    Object.defineProperty(SocketMgr, "instance", {
        get: function () {
            if (!this._instance) {
                this._instance = new SocketMgr();
            }
            return this._instance;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SocketMgr.prototype, "url", {
        get: function () {
            return true ? this.debugUrl : this.releaseUrl;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SocketMgr.prototype, "port", {
        get: function () {
            return true ? this.debugPort : this.releasePort;
        },
        enumerable: true,
        configurable: true
    });
    /**
 * 发送protobuf消息
 * @param {number} protoId 协议号
 * @param {ProtoClass} protoClass proto类
 * @param {ProtoSendFunc} func 对proto实例对象进行赋值处理的方法
 */
    SocketMgr.prototype.sendProtobufMsg = function (protoId, protoClass, func) {
        console.log("[Send To Server]protoId:" + protoId);
        var msg = new egret.ByteArray();
        if (protoClass) {
            var proto = new protoClass();
            if (func) {
                func(proto);
            }
            var writer = protoClass.encode(proto);
            msg.writeUnsignedShort(writer.len);
            msg.writeUnsignedShort(protoId);
            if (writer.len > 0) {
                msg._writeUint8Array(writer.finish());
            }
        }
        else {
            msg.writeUnsignedShort(0);
            msg.writeUnsignedShort(protoId);
        }
        this.socketClient.writeBytes(msg);
    };
    /**
     * 处理protobuf消息
     * @param {} msg proto字节数组
     * @param {ProtoClass} protoClass proto类，解析结果模板
     * @returns {} 返回对应proto类实例对象
     */
    SocketMgr.prototype.handleProtobufMsg = function (msg, protoClass) {
        if (msg == null || msg.length < 1) {
            console.error('byte数组长度小于1！');
            return undefined;
        }
        if (msg instanceof egret.ByteArray) {
            return protoClass.decode(msg.bytes);
        }
        else {
            return protoClass.decode(msg);
        }
    };
    SocketMgr.prototype.openSocket = function () {
        this.socketClient = new egret.WebSocket();
        this.socketClient.addEventListener(egret.ProgressEvent.SOCKET_DATA, this.onReceive, this);
        this.socketClient.addEventListener(egret.Event.CONNECT, this.onConnect, this);
        this.socketClient.addEventListener(egret.Event.CLOSE, this.onClose, this);
        this.socketClient.addEventListener(egret.IOErrorEvent.IO_ERROR, this.onError, this);
        this.socketClient.type = egret.WebSocket.TYPE_BINARY;
        this.socketClient.connect(this.url, this.port);
    };
    /**
 * 连接上服务器的回调
 */
    SocketMgr.prototype.onConnect = function () {
        console.log("已连上服务器！");
        this.connected = true;
        this.reconnectTimes = 0;
        // 心跳
        if (this.heartBeatTimer) {
            this.heartBeatTimer.stop();
        }
        this.heartBeatTimer = new egret.Timer(5000, 0);
        this.heartBeatTimer.addEventListener(egret.TimerEvent.TIMER, this.onHeartBeat, this);
        this.heartBeatTimer.start();
        this.connectedCBs.forEach(function (v) {
            v.callback.call(v.thisObj);
        });
        setTimeout(function () {
            GameMgr.instance.dispatchEvent(new SocketEvent(SocketEvent.RECONNECT_MSG));
        }, 0);
        if (this.reconnecting) {
            this.reconnecting = false;
        }
        UserMgr.instance.sendUserinfo();
    };
    SocketMgr.prototype.onReceive = function (ev) {
        var byteArr = new egret.ByteArray();
        this.socketClient.readBytes(byteArr);
        var socketEv = new SocketEvent(SocketEvent.PROTO_MSG);
        socketEv.length = byteArr.readUnsignedShort();
        socketEv.protoId = byteArr.readUnsignedShort();
        console.log("[Receive From Server]protoId:" + socketEv.protoId);
        if (socketEv.protoId == Protocol.heartBeat) {
            this.heartBeatTimes = 0;
        }
        if (socketEv.length > 0) {
            // 注：客户端不需要处理断包粘包问题
            socketEv.message = new egret.ByteArray();
            byteArr.readBytes(socketEv.message, 0, socketEv.length); //byteArr.readAvailable);
        }
        GameMgr.instance.dispatchEvent(socketEv);
    };
    SocketMgr.prototype.onClose = function () {
        console.log("socket closed!");
        if (this.connected) {
            this.connected = false;
            this.heartBeatTimer.stop();
            this.heartBeatTimer = null;
        }
        this.reconnect();
    };
    SocketMgr.prototype.reconnect = function () {
        this.reconnectTimes++;
        if (this.reconnectTimes >= 3) {
            console.log('网络状况较差，请检查网络');
            return;
        }
        console.log("Reconnect login!");
        this.connectedCBs = [];
        this.errorCBs = [];
        this.reconnecting = true;
        this.socketClient.removeEventListener(egret.ProgressEvent.SOCKET_DATA, this.onReceive, this);
        this.socketClient.removeEventListener(egret.Event.CONNECT, this.onConnect, this);
        this.socketClient.removeEventListener(egret.Event.CLOSE, this.onClose, this);
        this.socketClient.removeEventListener(egret.IOErrorEvent.IO_ERROR, this.onError, this);
        this.socketClient = null;
        this.openSocket();
    };
    /**
 * 连接发生错误
 */
    SocketMgr.prototype.onError = function (res) {
        // console.error("网络连接异常中断");
        // this.errorCBs.forEach((v) => {
        // 	v.callback.call(v.thisObj, res.errMsg);
        // });
        // this.onClose();
    };
    SocketMgr.prototype.onHeartBeat = function () {
        if (this.heartBeatTimes >= 1) {
            this.onClose();
            this.heartBeatTimes = 0;
        }
        else {
            this.sendProtobufMsg(Protocol.heartBeat);
            this.heartBeatTimes++;
        }
    };
    /**
     * 添加连上服务器监听
     */
    SocketMgr.prototype.addConnectListener = function (callback, thisObj) {
        this.connectedCBs.push({ callback: callback, thisObj: thisObj });
    };
    /**
     * 添加连接错误监听
     */
    SocketMgr.prototype.addErrorListener = function (callback, thisObj) {
        this.errorCBs.push({ callback: callback, thisObj: thisObj });
    };
    return SocketMgr;
}());
__reflect(SocketMgr.prototype, "SocketMgr");
