declare interface ChipConfig {
	value: number,
	color: number,
	bg: string
}
const chipConfig: ChipConfig[] = [
	{ value: 0.1, color: 0x93100e, bg: 'chip_r_t' },
	{ value: 0.5, color: 0x126ae4, bg: 'chip_b_t' },
	{ value: 1, color: 0x7630a6, bg: 'chip_p_t' },
	{ value: 5, color: 0x196b31, bg: 'chip_b_t' },
	{ value: 10, color: 0x7f781b, bg: 'chip_y_t' },
	{ value: 20, color: 0x6c431e, bg: 'chip_o_t' }
]
const CHIPSIZE = {
	SMALL: 1,
	LARGE: 2
}
class UI_Bet extends UIBaseComponent {
	private gtTipsGroup: eui.Group;
	private gtTipsBtn: eui.Button;
	private gtTips1: eui.Label;
	private gtTips2: eui.Label;

	private chipGroup: eui.Group;
	private radioGroup: eui.RadioButtonGroup;
	private chipElements: eui.RadioButton[] = [];
	private clearBtn: eui.Button;
	private confirmBtn: eui.Button;
	private autoConfirmBtn: eui.Button;

	private thisTimeBet: eui.Label;
	private betEosText: eui.Label;
	private thisPhaseBet: eui.Label;
	private thisPhaseBetEosText: eui.Label;
	private getBetGtLangText: eui.Label;
	private getBetGtText: eui.Label;
	private getBetGtLangText0: eui.Label;
	private getBetGtText0: eui.Label;

	private chipNodePool: UI_ChipItem[] = [];
	public constructor() {
		super();
	}

	/**
	 * 绑定事件
	 */
	protected bindEvents(): void {
		super.bindEvents();
		mouse.setButtonMode(this.gtTipsBtn, true);
		mouse.setButtonMode(this.clearBtn, true);
		mouse.setButtonMode(this.confirmBtn, true);
		mouse.setButtonMode(this.autoConfirmBtn, true);
		this.clearBtn.addEventListener(egret.TouchEvent.TOUCH_TAP, this.onClearBtn, this);
		this.confirmBtn.addEventListener(egret.TouchEvent.TOUCH_TAP, this.onConfirmBtn, this);
		this.autoConfirmBtn.addEventListener(egret.TouchEvent.TOUCH_TAP, this.onAutoConfirmBtn, this);
	}

	/**
	 * 创建完成时调用
	 */
	protected onCreate(): void {
		super.onCreate();
		this.render();
		this.onLangChange();
	}

	/**
	 * 渲染UI时调用
	 */
	protected render(): void {
		this.initRadioButtonWithGroup();
	}

	private onClearBtn(ev: egret.TouchEvent): void {
		AudiosMgr.instance.playButtonClick();
		this.clearChip();
	}

	private clearChip(): void {
		this.chipNodePool.forEach(node => {
			node.parent.removeChild(node);
		})
		Bet.instance.setChipNodes(this.chipNodePool);
		this.chipNodePool = [];
	}

	//TODO user confirm bet
	private onConfirmBtn(ev: egret.TouchEvent): void {
		AudiosMgr.instance.playButtonClick();

		if (UserMgr.instance.isLogin()) {
			console.log('confirm bet');
		}
	}

	private onAutoConfirmBtn(ev: egret.TouchEvent): void {
		AudiosMgr.instance.playButtonClick();

		if (UserMgr.instance.isLogin()) {
			Auto.instance.openAutoView();
		}
	}

	private initRadioButtonWithGroup(): void {
		this.radioGroup = new eui.RadioButtonGroup();
		this.radioGroup.addEventListener(eui.UIEvent.CHANGE, this.radioChangeHandler, this);
		chipConfig.forEach((config, index) => {
			let rdb: eui.RadioButton = new eui.RadioButton();
			rdb.skinName = chipItem;
			rdb.label = config.value.toString();
			(<eui.Label>rdb['labelDisplay']).textColor = config.color;
			(<eui.Image>rdb['chipItemImage']).source = config.bg;
			rdb.value = config.value;
			mouse.setButtonMode(rdb, true);
			rdb.group = this.radioGroup;
			rdb.x = 10 + rdb.width * index;
			this.chipElements.push(rdb);
			this.chipGroup.addChild(rdb);
		})
	}
	private radioChangeHandler(evt: eui.UIEvent): void {
		AudiosMgr.instance.playButtonClick();

		var radioGroup: eui.RadioButtonGroup = evt.target;
		this.chipElements.forEach(radio => {
			egret.Tween.removeTweens(radio);
			if (radio == radioGroup.selection) {
				if (radio.y == -30) return;
				egret.Tween.get(radio).to({ y: -40 }, 250, egret.Ease.sineIn);
			} else {
				if (radio.y == 0) return;
				egret.Tween.get(radio).to({ y: 0 }, 250, egret.Ease.sineIn);
			}
		})
		Bet.instance.chipValue = radioGroup.selectedValue;
	}

	getLocalPoint(childNode: egret.DisplayObjectContainer): egret.Point {
		let x: number = childNode.width / 2;
		let y: number = childNode.height / 2;
		while (childNode != this) {
			x += childNode.x;
			y += childNode.y;

			childNode = childNode.parent;
		}

		return new egret.Point(x, y);
	}

	chipIn(contaniner: egret.DisplayObjectContainer, stageX: number, stageY: number, chipNode: UI_ChipItem): void {
		let start = this.radioGroup.selection;
		let localStart = start.localToGlobal(0, 0);
		let localEnd = new egret.Point(stageX, stageY);

		contaniner.addChild(chipNode);
		this.chipNodePool.push(chipNode);
		chipNode.x = localStart.x;
		chipNode.y = localStart.y;

		egret.Tween.get(chipNode).to({ x: localEnd.x, y: localEnd.y }, 300, egret.Ease.sineIn);
	}

	//TODO render gamestart
	renderGameStart(): void {
		this.confirmBtn.enabled = true;
		this.clearChip();
	}

	//TODO render lotteryInfo
	renderLotteryInfo(): void {
		this.confirmBtn.enabled = false;
		if (this.radioGroup.selectedValue) {
			egret.Tween.get(this.radioGroup.selection).to({ y: 0 }, 250, egret.Ease.sineIn);
			this.radioGroup.selectedValue = null;
		}
	}


	/**
	 * 改变语言时调用
	 */
	protected onLangChange(ev?: egret.Event): void {
		let {Bet} = Lang.instance.lang();

		this.clearBtn.label = Bet.clear;
		this.confirmBtn.label = Bet.confirmBet;
		this.autoConfirmBtn.label = Bet.autoBet;
		this.thisTimeBet.text = Bet.thisTime;
		this.thisPhaseBet.text = Bet.thisPhase;
		this.getBetGtLangText.text = this.getBetGtLangText0.text = Bet.getBetGtLangText;
		this.gtTips1.text = Bet.gtTips1;
		this.gtTips2.text = Bet.gtTips2;
	}

	/**
	 * 界面关闭时调用
	 */
	public onClose(): void {
	}
}